/*
 * The code is a simplified version of the code by Matthias Neeracher, in turn based on
 * the code by Nathan Seidle published on the Sparkfun website. See the original comments
 * below.
 *
 * In comparison with the original,
 * - the code no longer supports the UART communication since SPI is needed in any case to
 *   set VS1053 into realtime-midi mode (at least on common breakouts).
 * - it includes a function setInstrument(channel, instrument).
 *
 * The library assumes the following wiring:
 *
 * Vcc -> 5V
 * GND -> GND
 * 6 -> XCS
 * 7 -> XDSC
 * 8 -> RST (aka RT, XRST)
 * 9 -> DREQ
 * 11 -> SI (aka MOSI)
 * 12 -> SO (aka MISO)
 * 13 -> SCK
 *
 * Pins 11-13 must be used for SI (MOSI), SO (MISO) and SCK.
 * Pins 6-9 can be changed; in this case, modify the below #define's.
 */
/*
 *
 * Author:   Matthias Neeracher
 *
 * This code is in the public domain, with the exception of the contents of sVS1053b_Realtime_MIDI_Plugin.
 *
 * The code is based on Nathan Seidle's Sparkfun Electronics example code for the Sparkfun 
 * MP3 Player and Music Instrument shields and and VS1053 breakout board. 
 *
 * http://www.sparkfun.com/Code/MIDI_Example.pde
 * http://dlnmh9ip6v2uc.cloudfront.net/datasheets/Dev/Arduino/Shields/VS_Shield_Example.zip
 *
 *      Spark Fun Electronics 2011
 *      Nathan Seidle
 *
 *      This code is public domain but you buy me a beer if you use this and we meet someday (Beerware license).
 *
 * THEORY OF OPERATIONS
 *
 * The VS1053b has two ways of playing MIDI: One method is that you simply send a Standard MIDI level 0 file through
 * SPI, and the chip will play it. This works exactly the same way as MP3 mode and will not be discussed further here.
 * The other method is that the VS1053b has a "Real Time MIDI mode", in which it will instantly execute MIDI commands 
 * sent to it through either the UART or SPI. 
 *
 * Real Time MIDI mode can be enabled with two different methods, controlled by USE_GPIO_INIT 
 *  (1) Setting GPIO1 to HIGH (which is hardwired in the Sparkfun Music Instrument shield, and can be done through 
 *      pin 4 in the MP3 Player Shield)
 *  (0) Sending a small software patch through SPI.
 *
 * MIDI data can be sent with two different methods as well, controlled by USE_SPI_MIDI
 *  (0) Through a (software) serial connection on pin 3, at 31250 baud
 *  (1) Through SPI, at an arbitrary data rate. For SPI, each byte of MIDI data needs to be prefixed by a 0 byte 
 *      (The V1053b data sheet erroneously states that the padding should be a 0xFF byte).
 *
 * Both initialization methods and both transmission methods can be selected through the #defines below. Out of the box, 
 * it probably makes most sense to enable real time MIDI through pin 4, and send serial data through pin 3, but if you 
 * want to cut the traces for pin 3 and 4 and use those pins for another purpose, the alternative methods may come in 
 * handy. 
 */ 

#ifndef MIDI_HPP
#define MIDI_HPP

#include <SPI.h>

#define VS_XCS    6 // Control Chip Select Pin (for accessing SPI Control/Status registers)
#define VS_XDCS   7 // Data Chip Select / BSYNC Pin
#define VS_RESET  8 //Reset is active low
#define VS_DREQ   9 // Data Request Pin: Player asks for more data

void init_midi();
void sendMIDI(byte data);
void talkMIDI(byte cmd, byte data1, byte data2=0);
void noteOn(byte channel, byte note, byte attack_velocity=127);
void noteOff(byte channel, byte note, byte release_velocity=127);
void setInstrument(byte channel, byte instrument);
#endif